import javax.swing.JPanel;
import javax.swing.JOptionPane;

import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Line2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.BasicStroke;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import graph.*;

@SuppressWarnings("serial")
public class GraphCanvas extends JPanel implements MouseListener,
		MouseMotionListener {
	private boolean insertVertex, editGraph;
	private Graph bspGraph;
	String vname = "A";
	private int altMouseX, altMouseY;
	private HashMap<String, Shape> shapes = new HashMap<String, Shape>();
	private DijkstraVertex vertexMousePressed = null;
	private boolean pause = false;

	public GraphCanvas(Graph graph) {
		super();
		insertVertex = false;
		editGraph = false;
		this.bspGraph = graph;
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	public boolean getPause() {
		return pause;
	}

	public void setPause(boolean fPause) {
		pause = fPause;
	}

	public void toggleInsertVertex() {
		insertVertex = !insertVertex;
		deleteAllMarks();
		editGraph = false;
	}

	public void toggleEditGraph() {
		editGraph = !editGraph;
		insertVertex = false;
		if (!editGraph) {
			deleteAllMarks();
			repaint();
		}
	}

	public void resetVName() {
		vname = "A";
	}

	public void setGraph(Graph graf) {
		bspGraph = graf;
		List<Vertex> vList = bspGraph.getVertices();
		shapes.clear();
		vList.toFirst();
		char max = 'A';
		while (vList.hasAccess()){
			DijkstraVertex vp = (DijkstraVertex)vList.getContent();
			int x = vp.getX();
			int y = vp.getY();
			String id = vp.getID();
			shapes.put(id, new Ellipse2D.Double(x - 13, y - 13,26, 26));
			char z = id.charAt(0);
			if (z>max) {
				max = z;
			}
			vList.next();		
		}	
		max++;
		vname = ""+max;
	}

	public void deleteAllMarks() {
		bspGraph.setAllVertexMarks(false);
		bspGraph.setAllEdgeMarks(false);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D) g;

		paintEdges(g2d);
		paintVertices(g2d);
	}

	public void paintVertices(Graphics2D g2d) {
		List<Vertex> vList = bspGraph.getVertices();
		vList.toFirst();
		DijkstraVertex v;
		while (vList.hasAccess()) {
			v = (DijkstraVertex) vList.getContent();
			int x = v.getX();
			int y = v.getY();
			String id = v.getID();
			g2d.setColor(Color.white);
			g2d.fill(shapes.get(id));
			if (v.isMarked()) {
				g2d.setStroke(new BasicStroke(3));
				g2d.setColor(Color.red);
			} else {
				g2d.setStroke(new BasicStroke(1));
				g2d.setColor(Color.black);
			}
			g2d.draw(shapes.get(id));
			g2d.drawString(id, x - 4, y + 4);
			vList.next();
		}
		;
	}

	public void paintEdges(Graphics2D g2d) {
		List<Edge> eList = bspGraph.getEdges();
		if (!eList.isEmpty()) {
			eList.toFirst();
			Edge e;
			while (eList.hasAccess()) {
				e = (Edge) eList.getContent();
				Vertex[] v1 = e.getVertices();
				int x1 = ((DijkstraVertex) v1[0]).getX();
				int y1 = ((DijkstraVertex) v1[0]).getY();
				int x2 = ((DijkstraVertex) v1[1]).getX();
				int y2 = ((DijkstraVertex) v1[1]).getY();
				if (e.isMarked()) {
					g2d.setStroke(new BasicStroke(3));
					g2d.setColor(Color.red);
				} else {
					g2d.setStroke(new BasicStroke(1));
					g2d.setColor(Color.black);
				}
				Line2D l2d = new Line2D.Double(x1, y1, x2, y2);
				g2d.draw(l2d);
				Polygon pol = new Polygon();
				pol.addPoint(x1 - 7, y1 - 7);
				pol.addPoint(x2 - 7, y2 - 7);
				pol.addPoint(x2 + 7, y2 + 7);
				pol.addPoint(x1 + 7, y1 + 7);
				String key = v1[0].getID() + v1[1].getID();
				shapes.put(key, pol);
				paintWeight(g2d, e.getWeight(), x1 + (x2 - x1) / 2, y1
						+ (y2 - y1) / 2, key);
				eList.next();
			}
		}
	}

	public void paintWeight(Graphics2D g2d, double w, double x, double y,
			String key) {
		String wString = "" + w;
		FontMetrics fm = g2d.getFontMetrics();
		double width = fm.stringWidth(wString);
		double height = fm.getHeight();
		Rectangle2D r2d = new Rectangle2D.Double(x - width / 2 - 2, y - height
				/ 2 - 2, width + 4, height + 4);
		key = key + "W";
		shapes.put(key, r2d);
		g2d.setColor(Color.white);
		g2d.fill(r2d);
		g2d.setColor(Color.black);
		g2d.setStroke(new BasicStroke(1));
		g2d.setColor(Color.black);
		g2d.draw(r2d);
		g2d.drawString(wString, (int) (x - width / 2),
				(int) (y + height / 2 - 3));
	}

	public DijkstraVertex searchVertex(int x, int y) {
		List<Vertex> vList = bspGraph.getVertices();
		vList.toFirst();
		DijkstraVertex v;
		boolean found = false;
		v = null;
		while (vList.hasAccess() && !found) {
			v = (DijkstraVertex) vList.getContent();
			if (v.contains(x, y)) {
				found = true;
			} else
				vList.next();
		}
		if (found)
			return v;
		else
			return null;
	}

	public Edge searchEdge(int x, int y) {
		List<Edge> eList = bspGraph.getEdges();
		eList.toFirst();
		boolean found = false;
		Edge e = null;
		while (eList.hasAccess() && !found) {
			e = eList.getContent();
			Vertex[] vertices = e.getVertices();
			String key = vertices[0].getID() + vertices[1].getID();
			Shape sh = shapes.get(key);
			if (sh.contains(x, y)) {
				found = true;
			} else
				eList.next();
		}
		if (found)
			return e;
		else
			return null;
	}

	public Edge searchWeight(int x, int y) {
		List<Edge> eList = bspGraph.getEdges();
		eList.toFirst();
		boolean found = false;
		Edge e = null;
		Shape sh = null;
		while (eList.hasAccess() && !found) {
			e = eList.getContent();
			Vertex[] vertices = e.getVertices();
			String key = vertices[0].getID() + vertices[1].getID();
			key = key + "W";
			sh = shapes.get(key);
			if (sh.contains(x, y)) {
				found = true;
			} else
				eList.next();
		}
		if (found)
			return e;
		else
			return null;
	}

	public void loesche() {
		List<Vertex> vList = bspGraph.getVertices();
		vList.toFirst();
		Vertex v;
		while (vList.hasAccess()) {
			v = vList.getContent();
			if (v.isMarked()) {
				bspGraph.removeVertex(v);
			}
			vList.next();
		}
		List<Edge> eList = bspGraph.getEdges();
		eList.toFirst();
		Edge e;
		while (eList.hasAccess()) {
			e = eList.getContent();
			if (e.isMarked()) {
				bspGraph.removeEdge(e);
			}
			eList.next();
		}
		repaint();
	}

	public double newWeight(String id1, String id2, double altGewicht) {
		double dGewicht = altGewicht;
		String gewicht = JOptionPane.showInputDialog(null,
				"Gewicht der Kante von " + id1 + " nach " + id2, "Gewicht",
				JOptionPane.PLAIN_MESSAGE);
		if (gewicht != null) {

			try {
				dGewicht = Double.parseDouble(gewicht);
			} catch (NumberFormatException exc) {
				dGewicht = altGewicht;
				System.out.println("Kein gültiges Gewicht");
			}
		}
		return dGewicht;
	}

	public void neueKanten() {
		List<Vertex> vList = bspGraph.getVertices();
		vList.toFirst();
		while (vList.hasAccess()) {
			Vertex vBegin = (Vertex) vList.getContent();
			if (vBegin.isMarked()) {
				List<Vertex> v2List = bspGraph.getVertices();
				v2List.toFirst();
				while (v2List.hasAccess()) {
					Vertex vEnd = (Vertex) v2List.getContent();
					boolean hasEdge = bspGraph.getEdge(vBegin, vEnd) != null;
					if (vEnd.isMarked() && vBegin != vEnd && !hasEdge) {
						Edge newEdge = new Edge(vBegin, vEnd, 0);
						bspGraph.addEdge(newEdge);
						newEdge.setWeight(newWeight(vBegin.getID(),
								vEnd.getID(), 0));
					}
					v2List.next();
				}
			}
			vList.next();
		}
		repaint();
	}

	public void mouseClicked(MouseEvent e) {
		if (insertVertex) {
			DijkstraVertex v = new DijkstraVertex(vname, e.getX(), e.getY());
			bspGraph.addVertex(v);
			Ellipse2D el = new Ellipse2D.Double(e.getX() - 13, e.getY() - 13,
					26, 26);
			shapes.put(vname, el);
			char z = vname.charAt(0);
			z++;
			vname = "" + z;
			repaint();
		} else if (editGraph) {
			DijkstraVertex v = searchVertex(e.getX(), e.getY());
			if (v != null) {
				if (v.isMarked())
					v.setMark(false);
				else
					v.setMark(true);
				repaint();
			} else {
				Edge edge = searchWeight(e.getX(), e.getY());
				if (edge != null) {
					Vertex[] vertices = edge.getVertices();
					edge.setWeight(newWeight(vertices[0].getID(),
							vertices[1].getID(), edge.getWeight()));
					repaint();
				} else {
					edge = searchEdge(e.getX(), e.getY());
					if (edge != null) {
						if (!edge.isMarked()) {
							edge.setMark(true);
						} else {
							edge.setMark(false);
						}
						repaint();
					}
				}

			}
		}
	}

	public void mouseReleased(MouseEvent e) {
		vertexMousePressed = null;
		repaint();
	}

	public void mousePressed(MouseEvent e) {
		altMouseX = e.getX();
		altMouseY = e.getY();
		vertexMousePressed = this.searchVertex(altMouseX, altMouseY);
	}

	public void mouseEntered(MouseEvent e) {

	}

	public void mouseExited(MouseEvent e) {

	}

	public void mouseDragged(MouseEvent e) {
		if (vertexMousePressed != null) {
			int x = e.getX();
			int y = e.getY();
			String id = vertexMousePressed.getID();
			Shape s = shapes.get(id);
			Rectangle2D r2d = s.getBounds();
			int neuX = vertexMousePressed.getX() + x - altMouseX;
			int neuY = vertexMousePressed.getY() + y - altMouseY;
			s = new Ellipse2D.Double(neuX - 13, neuY - 13, r2d.getWidth(),
					r2d.getHeight());
			vertexMousePressed.setXY(neuX, neuY);
			altMouseX = x;
			altMouseY = y;
			shapes.put(id, s);
			repaint();
		}
	}

	public void mouseMoved(MouseEvent e) {

	}

	public void pause() {
		/*
		 * synchronized(this) { int verzoegerung = 2000; while (getPause()){
		 * verzoegerung = 0; }
		 */
		try {
			repaint();
			Thread.sleep(1000);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
